/**
 * User interface for the Reaction Timer application
 */
package edu.msoe.se1021.reactiontimer.YOURLOGIN;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import edu.msoe.se1021.audio.WavPlayer;

/**
 * @author hornick
 * This class implements the UI for the Reaction Timer 
 */
@SuppressWarnings("serial") // don't warn about serialization issues
public class ReactionTimerUI extends JFrame {

	private EventHandler eh; // the object that processes event notifications; instance of the inner class

	private JLabel msgLabel; // text telling the user which button to press
	private JLabel statusLabel; // text telling the user the result of the last press
	private JLabel scoreLabel; // text displaying the score

	private JButton leftButton; // the left button
	private JButton middleButton; // the middle button
	private JButton rightButton; // the right button
	private String targetButton; // the button that should be pressed

	private javax.swing.Timer timer; 
	// note: there are also Timer classes in other packages that
	//  are completely different types of Timers, so we're explicit
	//  in specifying the Timer from javax.swing here.

	private List<ColorPlus> colors; // reference to a collection of random colors
	private int counter; // number of attempts
	private int score; 

	private long startClock; // System clock time at the time the "press button" signal was given 
	private long endClock; // System clock time at which a button was pressed (or a timeout occurred)
	private long reactionTime; // total reaction time (sum of all delays between startClock and endClock

	private static final int TIMEOUT = 2500; // timeout in ms, decrease for greater challenge
	private static final int MAX_ATTEMPTS = 10; // maximum number of attempts per game
	private static final int SCORE_POINTS = 10; // points awarded for each correct answer
	private static final int FAIL_PENALTY = 7; // points deducted for each incorrect answer
	private static final int TIMEOUT_PENALTY = 4; // points deducted for indecision (timeout before decision was made)


	/**
	 * constructor for the GUI class
	 * creates the JFrame window and subcomponents
	 */
	public ReactionTimerUI() {
		// initialize the collection of colors
		colors = new ArrayList<ColorPlus>();
		colors.add(new ColorPlus(Color.CYAN, "blue"));
		colors.add(new ColorPlus(Color.RED, "red"));
		colors.add(new ColorPlus(Color.GREEN, "green"));

		counter = 0; // initialize the button press counter
		score = 0; // initialize the score
		reactionTime = 0; // initial the user's total reaction time

		eh = new EventHandler(); // create an instance of the inner event handler class
		timer = new javax.swing.Timer(TIMEOUT, eh); //create the timer (but don't start it)
		createUI(); // create the UI
	}

	/**
	 * creates and initializes the UI
	 */
	private void createUI() {

		// specify what the "close" button in the title bar does 
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		setBounds(100, 100, 400, 400); // size and location of this UI on screen
		setResizable(true); // make it sizable
		setTitle("Reaction Timer");
		Font buttonFont = new Font("Arial", Font.BOLD, 24 ); //create a new font for the buttons
		Font labelFont = new Font("Arial", Font.BOLD, 18 ); //create a new font for the labels


		// The Container is what we actually add Components to...
		Container canvas = getContentPane();

		// specify how the Components will appear on the canvas
		FlowLayout fl = new FlowLayout(FlowLayout.LEFT);
		canvas.setLayout(fl); // use layout manager

		// create some buttons and a label
		leftButton = new JButton(""); // initial text on this button
		leftButton.setFont(buttonFont);
		leftButton.setActionCommand("left"); // permanent name for this button
		leftButton.setBackground(Color.DARK_GRAY);
		leftButton.setPreferredSize(new Dimension(100,100));

		middleButton = new JButton("");
		middleButton.setFont(buttonFont);
		middleButton.setActionCommand("middle");
		middleButton.setBackground(Color.DARK_GRAY);
		middleButton.setPreferredSize(new Dimension(100,100));

		rightButton = new JButton("");
		rightButton.setFont(buttonFont);
		rightButton.setActionCommand("right");
		rightButton.setBackground(Color.DARK_GRAY);
		rightButton.setPreferredSize(new Dimension(100,100));

		msgLabel = new JLabel(""); // displays the directive telling the user what button to press
		msgLabel.setFont(labelFont);
		msgLabel.setPreferredSize(new Dimension(200,25));
		msgLabel.setBackground(Color.BLACK);

		scoreLabel = new JLabel(""); // displays the current score
		scoreLabel.setFont(labelFont);
		scoreLabel.setPreferredSize(new Dimension(200,25));
		scoreLabel.setForeground(Color.BLACK);

		statusLabel = new JLabel(""); // displays success/failure/timeout status
		statusLabel.setFont(labelFont);
		statusLabel.setPreferredSize(new Dimension(200,25));
		statusLabel.setForeground(Color.BLACK);

		// add the Components to the Container
		canvas.add(leftButton);
		canvas.add(middleButton);
		canvas.add(rightButton);
		canvas.add(msgLabel);
		canvas.add(scoreLabel);
		canvas.add(statusLabel);

		// listen for events (button presses)
		leftButton.addActionListener( eh ); // subscribes the listener to this button's action events
		middleButton.addActionListener( eh );
		rightButton.addActionListener( eh );

		timer = new javax.swing.Timer(TIMEOUT, eh); // create the Timer
		timer.setActionCommand("countdown"); // friendly name for the Timer

		randomize(); // assign the JButtons random colors and text, and start the Timer

	} // end of createUI

	// This method randomizes the colors in the color collection and then
	// uses the first color as the color of the text telling the user which button
	// to press. Next, it re-randomizes the colors, and 
	// assigns the first three colors to the buttons, in order. 
	// It then generates a random number from 1 to 3 and uses that number 
	// to tell the user which button to press. 
	private void randomize() {
		Collections.shuffle(colors); // shuffle the colors
		// set the button text to the first three random color names
		leftButton.setText(colors.get(0).getName());
		middleButton.setText(colors.get(1).getName());
		rightButton.setText(colors.get(2).getName());

		// Use a random number generator to select one of the first three colors
		Random selector = new Random();
		int index = selector.nextInt(3); // random value from 0 to 2
		msgLabel.setForeground(colors.get(index));

		Collections.shuffle(colors); // shuffle again
		// set the text color to the first three random colors
		leftButton.setForeground(colors.get(0));
		middleButton.setForeground(colors.get(1));
		rightButton.setForeground(colors.get(2));

		// Use a random number generator to select one of the first three colors
		index = selector.nextInt(3); // random value from 0 to 2
		msgLabel.setText(String.format("Press the %s button", colors.get(index).getName()) );

		// remember which button is supposed to be pressed
		if( index == 0 ) {
			targetButton = "left";
		}
		else if( index == 1 ) {
			targetButton = "middle";
		}
		else {
			targetButton = "right";
		}
		timer.restart(); // start/restart the Timer 
		startClock = System.currentTimeMillis(); // current System clock time at which the Timer is started 
	}

	// TODO: Create a nested inner EventHandler class that implements the ActionListener interface.

		// TODO: In the class constructor, initializes the WavPlayer objects to play the various
		// sounds indicating that a correct button was pressed, an incorrect button
		// was pressed, or the timeout period (with no button pressed) elapsed.


		// TODO: Implement the ActionListener interface method. Inside this method,
			// 1) When an event occurs; get its source (JButton or Timer)
			// 2) Stop the Timer
			// 3) Put the current System clock time (the time this ActionEvent occurred) in the endClock variable
			// 4) Compute the time between the start of the Timer (started in randomize) and the current time (this is the reaction time for one attempt)
			// 5) Accumulate the total reaction time in the reactionTime variable
			// 6a) Check the event source. See if the current event came from a JButton
				// If so, get the specific button's Action Command string,
				// Check to see if the button that was pressed was the expected button (for a correct choice).
				
					// If the correct button was pressed, increment the score by SCORE_POINTS,
					// play the "score" sound, and indicate a correct button press in the status label.

					// If the incorrect button was pressed, decrement the score by FAIL_PENALTY points,
					// play the "fail" sound, and indicate an incorrect button press.
			// 6b) Otherwise, the event came from the Timer, indicating a timeout (a button was not pressed in time).
				// Decrement the score by TIMEOUT_PENALTY points,
				// play the "timeout" sound, and indicate a timeout.
			// 7) Display the points in the score label.
			// 8) Increment the counter keeping track of the number of attempts.
			// 9a) If the number of attempts reaches MAX_ATTEMPTS,
			// 		Display a JOPtionPane message dialog displaying the total reaction time in seconds, and exit.
			// 9b) Otherwise, call randomize() to reshuffle the buttons.

} // end outer ReactionTimerUI class 
